<?php
/**
 * 配置存储组件的基础类文件
 * 
 * @author dongbeta <xii@dongbeta.com>
 * @link http://xii.googlecode.com
 * @copyright Copyright @ 2010 Xii Project
 * @license http://www.opensource.org/licenses/bsd-license.php
 */

/**
 * 存储和读取配置信息
 * 
 * @example
 * 
 * 新增或者更新配置：
 * Yii::app()->config->set('name', 'value');
 * 读取配置：
 * Yii::app()->config->get('name', 'default');
 * 
 * @author dongbeta <xii@dongbeta.com>
 * @version $Id$
 * @package package xii.components.config
 */
abstract class XConfig extends CApplicationComponent
{
    /**
	 * @var string 缓存前缀
	 */
	public $cachePrefix = 'Xii.Components.XDbConfig.';

	/**
	 * @var int 缓存周期，-1代表不缓存，0代表永远不会失效
	 */
	public $cachingDuration = 0;
	
	private $_config = array(); //存储已经读取过的配置
	
	/**
	 * 返回配置名对应的读取配置值
	 * @param string $name 配置名
	 * @param mixed $default 如果不存在的话，则返回此默认值
	 */
	public function get($name, $default = null)
	{
		if(isset($this->_config[$name]))
			return $this->_config[$name];

		list($category, $term) = explode('_', $name, 2);
		
		if($this->cachingDuration >= 0)
			$cache = Yii::app()->getCache()->get($this->cachePrefix.$category);
        
		if(!isset($cache) || !$cache instanceof CConfiguration || !isset($cache[$name])) {
			if ($this->find($name)) {
			    if ($this->cachingDuration >= 0) {
			        if (!isset($cache) || !$cache instanceof CConfiguration)
    			        $cache = new CConfiguration();
    			    $cache[$name] = $this->_config[$name];
    			    Yii::app()->getCache()->set($this->cachePrefix.$category, $cache);
			    }
			} else
			    return $default;
		}else
		    $this->_config[$name] = $cache[$name];
		return $this->_config[$name];
	}
	
	/**
	 * 设置配置项，并存储。此操作会删除相应分类的缓存。
	 * @param string $name 配置项名称
	 * @param mixed $value 配置项的值
	 * @return boolean 是否设置成功
	 */
	public function set($name, $value)
	{
	    $result = $this->exist($name)?$this->update($name, $value):$this->add($name, $value);
	    if ($result) {
	        $this->_config[$name] = $value;
		    if($this->cachingDuration >= 0)
		        $this->cleanCache($name);
	    }
	    return $result;
	}
	
	/**
	 * 返回一个虚拟模型，用于给其他需要CModel的类使用
	 * @param string $alias 字段配置文件别名地址
	 * @return XConfigVirtualModel 虚拟模型
	 */
	public function getVirtualModel($alias)
	{
		Yii::import('ext.model.XVirtualModel');
		Yii::import('application.components.config.XConfigVirtualModel');
		return new XConfigVirtualModel($alias);
	}
	
	/**
	 * 清除缓存
	 * @param string $name 要清除缓存的配置项
	 */
	public function cleanCache($name)
	{
		Yii::app()->getCache()->delete($this->cachePrefix.substr($name, 0, strpos($name, '_')));
	}
	
	/**
	 * 设置self::$_config的值
	 * @param string $name 配置名
	 * @param mixed $value 配置值
	 */
	protected function setValue($name, $value)
	{
	    $this->_config[$name] = $value;
	}
	
	/**
	 * 从数据存储处取出相应的值。
	 * 如果成功取得此值，必须要设置self::$_config
	 * @param string $name 配置名
	 * @return 是否查询成功
	 */
	abstract protected function find($name);
	
	/**
	 * 更新存储系统中的配置值
	 * @param $name 配置名
	 * @param $value 配置值
	 * @return boolean 是否更新成功
	 */
	abstract protected function update($name, $value);
	
	/**
	 * 在存储系统添加一项中的配置值
	 * @param $name 配置名
	 * @param $value 配置值
	 * @return boolean 是否添加成功
	 */
	abstract protected function add($name, $value);
	
	/**
	 * 判断是否存在
	 * @param $name string 配置名
	 * @param $value mixed 配置值
	 * @return boolean 是否存在
	 */
	abstract protected function exist($name);
}